#!/usr/bin/env python
import gtk, gobject
import gnome.ui
import gtk.glade
import os

from moviesClass import *
from languageClass import *
from movieUI import *


def run_dialog( text, parent=None, messagetype=gtk.MESSAGE_WARNING, buttonstype=gtk.BUTTONS_OK, extrabuttons=[]):
    """Run a dialog with text 'text'.
       Extra buttons are passed as tuples of (button label, response id).
    """
    d = gtk.MessageDialog(None,
        gtk.DIALOG_DESTROY_WITH_PARENT,
        messagetype,
        buttonstype,
        '<span weight="bold" size="larger">%s</span>' % text)
    if parent:
        d.set_transient_for(parent.widgetTree.get_widget("window").get_toplevel())
    for b,id in extrabuttons:
        d.add_button(b,id)
    d.set_has_separator(0)
    d.vbox.set_spacing(12)
    hbox = d.vbox.get_children()[0]
    hbox.set_spacing(12)
    d.image.set_alignment(0.5, 0)
    d.image.set_padding(12, 12)
    d.label.set_use_markup(1)
    d.label.set_padding(12, 12)
    ret = d.run()
    d.destroy()
    return ret


class MoviesUI:
    filename = None
    languages = []
    library = Movies()

    _OBJECT_COLUMN = 0
    _REFERENCE_COLUMN = 1
    _TITLE_COLUMN = 2
    _LANGUAGE_COLUMN = 3
    _QUALITY_COLUMN = 4
    _RATING_COLUMN = 5

    def __init__(self):
        gnome.init("movies", "1.0")
        self.widgetTree = gtk.glade.XML("movies.glade")

        destroy_dic = { "on_quit_button_clicked": self.quit_signal_handler,
                        "on_saveAs_button_clicked": self.saveAs_signal_handler,
                        "on_save_button_clicked": self.save_signal_handler,
                        "on_open_button_clicked": self.open_signal_handler,
                        "on_new_button_clicked": self.new_signal_handler,
                        "on_add_button_clicked": self.add_signal_handler,
                        "on_remove_button_clicked": self.remove_signal_handler,
                        "on_about_clicked" : self.about}
        self.widgetTree.signal_autoconnect(destroy_dic)

        ##-- prepare the list
        view = self.widgetTree.get_widget("listOfMovies")
        model = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING, gobject.TYPE_STRING, gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING)
        view.set_model(model)
        view.set_headers_clickable(gtk.TRUE)
        ##- add events
        view.connect("button-release-event", self.button_clicked)
        view.connect("button-press-event", self.button_clicked)

        ##-- add columns
        ##- object
        renderer_object = gtk.CellRendererText()
        column_object = gtk.TreeViewColumn("Object", renderer_object, text = self._OBJECT_COLUMN)
        view.append_column(column_object)
        column_object.set_visible(False)
        
        ##- reference
        renderer_reference = gtk.CellRendererText()
        column_reference = gtk.TreeViewColumn("Id", renderer_reference, text = self._REFERENCE_COLUMN)
        view.append_column(column_reference)

        ##- title
        renderer_title = gtk.CellRendererText()
        column_title = gtk.TreeViewColumn("Title", renderer_title, text = self._TITLE_COLUMN)
        view.append_column(column_title)

        ##- language
        #column_language = gtk.TreeViewColumn("Language", renderer_language, expand=False)
        column_language = gtk.TreeViewColumn()
        column_language.set_title("Language")
        renderer_language = gtk.CellRendererPixbuf()
        column_language.pack_start(renderer_language, expand=False)
        column_language.add_attribute(renderer_language, 'pixbuf', self._LANGUAGE_COLUMN )
        view.append_column(column_language)
        
        ##- quality
        renderer_quality = gtk.CellRendererText()
        column_quality = gtk.TreeViewColumn("Quality", renderer_quality, text = self._QUALITY_COLUMN)
        view.append_column(column_quality)

        ##- rating
        renderer_rating = gtk.CellRendererText()
        column_rating = gtk.TreeViewColumn("Rating", renderer_rating, text = self._RATING_COLUMN)
        view.append_column(column_rating)

        ##-- languages
        self.languages = Languages.read_from_file("languages/languages.xml")

        ##-- end of init
        self.set_status("application ready")

    def set_status(self,aMessage):
        self.widgetTree.get_widget("statusbar").pop(0)
        self.widgetTree.get_widget("statusbar").push(0,aMessage)

    def quit_signal_handler(self,arg):
        gtk.main_quit()
        return True


    def new_signal_handler(self,arg):
        ## test
        response = run_dialog('Are you sure to remove all movies ?',
                              parent = self,
                              buttonstype = gtk.BUTTONS_YES_NO)
        if response == gtk.RESPONSE_YES:
            self.library.clear()
            self.update()
            self.set_status("Empty library.")

        #m = unicode("hello", 'latin-1')
        #t = unicode("title", 'latin-1')
        #dialog = gtk.Dialog(t.encode('utf-8'), self.widgetTree.get_widget("window"), 0,
        #                    (gtk.STOCK_NO, gtk.RESPONSE_NO, gtk.STOCK_YES, gtk.RESPONSE_YES))
        #hbox = gtk.HBox(gtk.FALSE, 8)
        #hbox.set_border_width(8)
        #dialog.vbox.pack_start(hbox, gtk.FALSE, gtk.FALSE, 0)
        #stock = gtk.image_new_from_stock(
        #    gtk.STOCK_DIALOG_QUESTION,
        #    gtk.ICON_SIZE_DIALOG)
        #hbox.pack_start(stock, gtk.FALSE, gtk.FALSE, 0)
        #label = gtk.Label(m.encode('utf-8'))
        #hbox.pack_start(label, gtk.TRUE, gtk.TRUE, 0)
        #dialog.show_all()
        #response = dialog.run()
        #dialog.destroy()

        ##
        #self.library.clear()
        #self.update()
        #self.set_status("Empty library.")
        return True


    def open_signal_handler(self,arg):
        file_selector = gtk.FileSelection("Open...")
        file_selector.set_select_multiple(False)
        file_selector.complete("*.xml")
        selectedFile = None
        def use_file_selection(button, file_selector):
            selection = file_selector.get_filename()
            if os.path.isdir(selection):
                pass
            else:
                selectedFile = selection
                file_selector.destroy()
            self.open_file(selectedFile)

        def close_file_selection(button, file_selector):
            file_selector.destroy()
            self.set_status("Open canceled.")

        file_selector.ok_button.connect("clicked", use_file_selection, file_selector)
        file_selector.cancel_button.connect( 'clicked', close_file_selection, file_selector)
        file_selector.show()
        self.set_status("Open ...")
        return True

    def open_file(self, aFilename):
        self.filename = aFilename
        self.library = Movies.read_from_file(self.filename)
        self.update()
        self.set_status("Using file [%s]." % self.filename)
        

    def update(self):
        view = self.widgetTree.get_widget("listOfMovies")
        model = view.get_model()
        model.clear()
        for aMovie in self.library.get_list():
            iter = model.append()
            model.set(iter, self._OBJECT_COLUMN, aMovie)
            model.set(iter, self._REFERENCE_COLUMN, aMovie.reference)
            model.set(iter, self._TITLE_COLUMN, aMovie.title)
            model.set(iter, self._LANGUAGE_COLUMN, self.languages.get_with_code(aMovie.language.code).get_icon() )
            model.set(iter, self._QUALITY_COLUMN, str(aMovie.quality.value))
            model.set(iter, self._RATING_COLUMN, str(aMovie.rating.value))

 
    def save_signal_handler(self,arg):
        file = open(self.filename,'w')
        file.write(self.library.to_xml(leftOffset=0))
        file.close()
        self.set_status("File [%s] saved." % self.filename)
        return True

 
    def saveAs_signal_handler(self,arg):
        fselect = gtk.FileSelection( "Choose a file name...")
        fselect.set_transient_for(self.widgetTree.get_widget("window").get_toplevel() )
        response = fselect.run()
        if response != gtk.RESPONSE_OK:
            fselect.destroy()
            return False
        aFilename = fselect.get_filename()
        if os.path.exists(aFilename):
            response = run_dialog('"%s" exists!\nOverwrite?' % os.path.basename(aFilename),parent = self,buttonstype = gtk.BUTTONS_YES_NO)
            if response == gtk.RESPONSE_NO:
                fselect.destroy()
                return False
        self.filename = os.path.abspath(aFilename)
        fselect.destroy()
        self.save_signal_handler(None)
        return True


    def add_signal_handler(self,arg):
        aMovieUI = MovieUI()
        aMovieUI.set_interactive(True)
        aMovieUI.on_accept = self.add_movie
        aMovieUI.on_cancel = None


    def add_movie(self, aNewMovie):
        if len(aNewMovie.title) < 2:
            return False
        self.library.add(aNewMovie)
        self.update()
        self.set_status("Movie [%s] added." % aNewMovie.title)
        return True
        

    def remove_signal_handler(self,arg):
        selectedMovie = self.get_selected_movie()
        if selectedMovie is None:
            self.set_status("No movie selected.")
        else:
            self.library.remove(selectedMovie)
            movieTitle = selectedMovie.title
            self.update()
            self.set_status("Movie [%s] removed." % movieTitle)
        return True


    def button_clicked(self, widget, event):
        if (event.type is gtk.gdk._2BUTTON_PRESS) and (event.button == 1):
            self.edit_movie(self.get_selected_movie())
            return 
        if (event.type is gtk.gdk.BUTTON_RELEASE) and (event.button == 1):
            ##-- a movie is selected
            return 


    def get_selected_movie(self):
        (model, path) = self.widgetTree.get_widget("listOfMovies").get_selection().get_selected_rows()
        if len(path) is 0:
            return None
        ##-- get the first selection only
        iter = model[path[0]]
        return iter[self._OBJECT_COLUMN]

    def edit_movie(self, aMovie):
        aMovieUI = MovieUI()
        aMovieUI.set_interactive(True)
        aMovieUI.update_fields_from(self.get_selected_movie())
        aMovieUI.on_accept = self.update_movie

    def update_movie(self, aMovie):
        self.get_selected_movie().merge_with(aMovie)
        self.update()
        self.set_status("Movie [%s] updated." % aMovie.title)
        return True
        

    def about(self,args):
        dialog = gtk.MessageDialog(self.widgetTree.get_widget("window"), gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, "Movies by JC'06/2004".encode('utf-8'))
        dialog.set_default_response(gtk.BUTTONS_OK)
        dialog.connect('response', lambda dialog, response: dialog.destroy())
        dialog.show()

